package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.DetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 处理OFS销售订单，邮费金额分摊的逻辑
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：AmountAllocationUtil
 * @Date：2024/9/21 18:19
 * @Filename：AmountAllocationUtil
 */
@Component
public class OfsOrderSalePostageFeeAllocationUtil {

    Logger logger = LoggerFactory.getLogger(OfsOrderSalePostageFeeAllocationUtil.class);

    /**
     * TOC销售订单邮费分摊逻辑
     *
     * @author liuyang
     */
    public void batchTocSalesPostageFeeAllocation(List<HeaderDetailsDto> headerDetailsDtoList) {
        Assert.notNull(headerDetailsDtoList, "headerDetailsDtoList不能为空！");

        logger.info("==TOC销售邮费金额分摊开始==");
        headerDetailsDtoList.forEach(headerDetailsDto -> {
            try {
                tocSalesPostageFeeAllocation(headerDetailsDto);
            } catch (Exception e) {
                logger.error("tocSalesPostageFeeAllocation方法抛出异常，批量处理TOC销售邮费分摊异常", e);
            }
        });
        logger.info("==TOC销售邮费金额分摊结束==");
//        logger.info("=记录优惠金额分摊结果开始=");
//        printAmountOfMoneyNum(headerDetailsDtoList);
//        logger.info("=记录优惠金额分摊结果完成=");
    }

    /**
     * TOC邮费金额分摊
     *
     * @param headerDetailsDto OFS销售订单对象
     * @author liuyang
     */
    public void tocSalesPostageFeeAllocation(HeaderDetailsDto headerDetailsDto) throws Exception {
        Assert.notNull(headerDetailsDto, "OFS销售订单对象不能为空");

        HeaderDto header = headerDetailsDto.getHeader();
        List<DetailsDto> detailsOld = headerDetailsDto.getDetails();

        //拷贝一份List集合，并移除没有「实付金额」的OFS销售订单明细行，不参与分摊
        //运费：postageAmount
        List<DetailsDto> details = copyDetailsDto(detailsOld);
        details.removeIf(dto -> dto.getTotalPayAmount() == null || "0".equals(new BigDecimal(dto.getTotalPayAmount()).stripTrailingZeros().toPlainString()));

        //计算分摊百分比
        calculatePercentage(headerDetailsDto, details, header.getCode());

        //销售订单运费
        String postageAmount = header.getPostageAmount();

        //测试
//        postageAmount = "10";

        //运费可能没有，也有可能为0！
        BigDecimal postageAmountDiscounts = null;
        if (postageAmount != null && !"".equals(postageAmount) && !"0".equals(new BigDecimal(postageAmount).stripTrailingZeros().toPlainString())) {
            postageAmountDiscounts = new BigDecimal(postageAmount);
        }

        BigDecimal totalShareTargetPlatformDiscounts = new BigDecimal("0");
        //分摊金额=分摊比例*优惠金额
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);
            //分摊比例
            BigDecimal postageFeeSharingRatio = detailsDto.getPostageFeeSharingRatio();

            //这里应该先8为小数，实际汇总到出库单时，再四舍五入两位小数
            BigDecimal shareTargetPlatformDiscounts = null;
            if (postageFeeSharingRatio != null && postageAmountDiscounts != null) {
                shareTargetPlatformDiscounts = postageAmountDiscounts.multiply(postageFeeSharingRatio).setScale(2, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalFreightDiscounts(postageAmountDiscounts);
                detailsDto.setShareTargetFreightDiscounts(shareTargetPlatformDiscounts);

                totalShareTargetPlatformDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPlatformDiscounts);
            }

            //如果是最后一行，则把尾差进行追加
            if (i == details.size() - 1) {
                if (!"0".equals(totalShareTargetPlatformDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetFreightDiscounts() != null) {
                    //原始运费金额-sum（分摊金额）
                    BigDecimal subtract = postageAmountDiscounts.subtract(totalShareTargetPlatformDiscounts);
                    if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                        BigDecimal shareTargetFreightDiscounts = detailsDto.getShareTargetFreightDiscounts();
                        detailsDto.setShareTargetFreightDiscounts(shareTargetFreightDiscounts.add(subtract));
                    }
                }
            }

            //拷贝回到原始的集合
            copyDetailsDtoOld(details, detailsOld);
        }
    }

    /**
     * 计算百分比
     * 分摊比例=实付金额/sum(实付金额)
     *
     * @param headerDetailsDto2 OFS销售订单
     * @param details           克隆后的销售订单明细行
     * @param ofsCode           OFS销售订单编码
     * @author liuyang
     */
    private void calculatePercentage(HeaderDetailsDto headerDetailsDto2, List<DetailsDto> details, String ofsCode) throws Exception {
        Assert.notNull(headerDetailsDto2, "headerDetailsDto2不能为空");
        Assert.notNull(details, "details不能为空");
        Assert.notNull(ofsCode, "ofsCode不能为空");

        HeaderDto header1 = headerDetailsDto2.getHeader();
        List<DetailsDto> details1 = headerDetailsDto2.getDetails();

        BigDecimal sumTotalPayAmountBigDecimal = sumTotalPayAmountAmount(details1, header1.getCode());
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);

            String totalPayAmount = detailsDto.getTotalPayAmount();
            //如果实付金额为0则不进行分摊！
            if (totalPayAmount != null && !"0".equals(new BigDecimal(totalPayAmount).stripTrailingZeros().toPlainString())) {
                BigDecimal totalPayAmountBigDecimal = new BigDecimal(totalPayAmount);
                //得到分摊比例
                BigDecimal percentageBigDecimal = totalPayAmountBigDecimal.divide(sumTotalPayAmountBigDecimal, 20, BigDecimal.ROUND_HALF_UP).setScale(8, BigDecimal.ROUND_HALF_UP);
                if (details != null && details.size() > 0) {
                    for (int j = 0; j < details.size(); j++) {
                        DetailsDto detailsDto1 = details.get(j);
                        if (detailsDto1.getId() != null && detailsDto.getId() != null) {
                            if (detailsDto1.getId().equals(detailsDto.getId())) {
                                details.get(j).setPostageFeeSharingRatio(percentageBigDecimal);
                            }
                        }
                    }
                }
            }
            //logger.info打太多太慢了
//            else {
//                logger.info(totalPayAmount, "totalPayAmount(商品金额)为空！OFS销售订单号：{} 存货编码：{}", ofsCode, detailsDto.getSkuCode());
//            }
        }
    }

    /**
     * 累计这一单对应的实付金额
     *
     * @param details1 OFS销售订单明细行
     * @param code     OFS销售订单号
     * @author liuyang
     */
    private BigDecimal sumTotalPayAmountAmount(List<DetailsDto> details1, String code) throws Exception {
        Assert.notNull(details1, "details1不能为空");
        Assert.notNull(code, "code不能为空");

        BigDecimal totalPayAmountTotalBigDecimal = new BigDecimal("0");
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);
            //实付金额
            String totalPayAmount = detailsDto.getTotalPayAmount();
            if (totalPayAmount != null && !"0".equals(new BigDecimal(totalPayAmount).stripTrailingZeros().toPlainString())) {
                BigDecimal totalPayAmountBigDecimal = new BigDecimal(totalPayAmount);
                totalPayAmountTotalBigDecimal = totalPayAmountTotalBigDecimal.add(totalPayAmountBigDecimal);
            }
        }
        if ("0".equals(totalPayAmountTotalBigDecimal.stripTrailingZeros().toPlainString())) {
            logger.info("OFS销售订单号：{} 整单明细行全部的「实付金额」都为0，邮费都不参与分摊！", code);
        }
        return totalPayAmountTotalBigDecimal;
    }

    /**
     * 集合复制
     *
     * @param details 旧OFS销售订单明细行
     * @author liuyang
     */
    private List<DetailsDto> copyDetailsDto(List<DetailsDto> details) throws Exception {
        List<DetailsDto> newDetails = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);

            DetailsDto newDetailsDto1 = new DetailsDto();
            BeanUtil.copyPropertiesV2(detailsDto, newDetailsDto1);
            newDetails.add(newDetailsDto1);
        }
        return newDetails;
    }

    /**
     * 拷贝到原集合中
     *
     * @param newDetails 新OFS销售订单明细行
     * @param detailsOld 旧OFS销售订单明细行
     * @author liuyang
     */
    private void copyDetailsDtoOld(List<DetailsDto> newDetails, List<DetailsDto> detailsOld) throws Exception {
        if (newDetails != null && detailsOld != null) {
            for (int i = 0; i < detailsOld.size(); i++) {
                DetailsDto detailsDto = detailsOld.get(i);
                for (int j = 0; j < newDetails.size(); j++) {
                    DetailsDto detailsDto1 = newDetails.get(j);
                    if (detailsDto.getId().equals(detailsDto1.getId())) {
                        //把分摊后的金额拷贝到旧的OFS销售订单集合中
                        BeanUtil.copyPropertiesV2(detailsDto1, detailsDto);
                    }
                }
            }
        } else {
            logger.error("copyDetailsDtoOld方法newDetails或者detailsOld为空！");
        }
    }

    /**
     * 打印金额参数
     *
     * @param headerDetailsDtoList OFS销售订单号
     */
//    public void printAmountOfMoneyNum(List<HeaderDetailsDto> headerDetailsDtoList) {
//        try {
//            if (headerDetailsDtoList != null && headerDetailsDtoList.size() > 0) {
//                for (int i = 0; i < headerDetailsDtoList.size(); i++) {
//                    HeaderDetailsDto headerDetailsDto = headerDetailsDtoList.get(i);
//                    HeaderDto header = headerDetailsDto.getHeader();
//                    List<DetailsDto> details = headerDetailsDto.getDetails();
//
//                    logger.info("OFS销售订单号：{} 平台优惠：{} 商家优惠：{} 达人优惠：{} 支付优惠：{}", header.getCode(), header.getPlatformDiscounts(), header.getMerchantDiscounts(), header.getExpertDiscounts(), header.getPayDiscounts());
//                    if (details != null && details.size() > 0) {
//                        for (int j = 0; j < details.size(); j++) {
//                            DetailsDto detailsDto = details.get(j);
//
//                            logger.info("start================================");
//                            logger.info("OFS销售订单明细主键：{}", detailsDto.getId());
//                            if (detailsDto.getSharingRatio() != null) {
//                                logger.info("分摊比例：{}", detailsDto.getSharingRatio().stripTrailingZeros().toPlainString());
//                            } else {
//                                logger.info("分摊比例：无");
//                            }
//                            logger.info("商品金额：{}", detailsDto.getItemTotalAmount());
//                            if (detailsDto.getShareTargetPlatformDiscounts() != null) {
//                                logger.info("分摊平台优惠：{}", detailsDto.getShareTargetPlatformDiscounts().stripTrailingZeros().toPlainString());
//                            } else {
//                                logger.info("分摊平台优惠：无");
//                            }
//                            if (detailsDto.getShareTargetMerchantDiscounts() != null) {
//                                logger.info("分摊商家优惠：{}", detailsDto.getShareTargetMerchantDiscounts().stripTrailingZeros().toPlainString());
//                            } else {
//                                logger.info("分摊商家优惠：无");
//                            }
//                            if (detailsDto.getShareTargetExpertDiscounts() != null) {
//                                logger.info("分摊达人优惠：{}", detailsDto.getShareTargetExpertDiscounts().stripTrailingZeros().toPlainString());
//                            } else {
//                                logger.info("分摊达人优惠：无");
//                            }
//                            if (detailsDto.getShareTargetPayDiscounts() != null) {
//                                logger.info("分摊支付优惠：{}", detailsDto.getShareTargetPayDiscounts().stripTrailingZeros().toPlainString());
//                            } else {
//                                logger.info("分摊支付优惠：无");
//                            }
//                            logger.info("end================================");
//                        }
//                    }
//                }
//            }
//        } catch (Exception e) {
////            e.printStackTrace();
//            logger.error("printAmountOfMoneyNum方法抛出异常", e);
//        }
//    }
}